/**********************************************************************************************
Macro Name: UNI_LOGREG
Created Date/Author: Aug. 23, 2012/Sungjin Kim 
Last Update Date/Person: Jan 24, 2015/Yuan Liu 
Current Version: V14
Working Environment: SAS 9.3 English version

Contact: Dr. Yuan Liu yliu31@emory.edu

PURPOSE: To conduct univariate logistic regression for each variable in the dataset.  A 
proportional odds model is fitted for a binary outcome variable; a cumulative logit function is
used for an ordinal (ordered) outcome variable with 3 or more levels.  The odds ratio with 95%
CI is presented along with the p-value.  For categorical variables, the reference group will be
shown along with the number of observation in each category.     

NOTE:    This macro does not work with a generalized logit model with a nominal outcome. 

Parameters: 

DATASET        The name of the data set to be analyzed.
OUTCOME        The name of the outcome variable.  It must be binary or ordinal.  If ordinal
               then it should be a numeric variable.
EVENT          The event category for the binary response model (optional).  You can specify 
               the value in quotes.  This will be passed to the event= option in the model 
               statement.  Leave this blank if you have an ordinal outcome with more than 2 
               levels.
DESC           Set to T to reverse the order of an ordinal outcome (optional).  The order will be
               based on the internal order.  Only specify this if the EVENT parameter is blank.  
               The default value is F.
CLIST          List of categorical variables, separated by empty space. Each variable name must 
               be less than 31 characters long.
CREFLIST       List of the categorical variables with reference levels specified separated by * 
               (optional).    
NLIST          List of numerical variables, separated by empty space.
MATCHID		   If the data is from a matched sample, specify the match ID here. Otherwise leave it blank.
TYPE3          Set to F to suppress type III p-values from being reported in the table 
               (optional).  The default value is T.  This will only have an effect if 
               categorical variables are specified in CLIST.
FIRTH          Set to T to use Firths penalized maximum likelihood estimation to reduce bias
               in the parameter estimates.  The default value is F.
PO             Set to T to check the proportional odds assumption by reporting the score test
               p-values.  Only set to T if using an ordinal outcome.  The default value is F.  
DOC            Set to T to create a RTF file containing the output or F to suppress creation of 
               the RTF file (optional).  The default value is T.
OUTPATH        Path for output table to be stored.
FNAME          File name for output table.
ORIENTATION    Value of PORTRAIT or LANDSCAPE to indicate the paper layout of the report 
               (optional).  The default value is PORTRAIT.  
DEBUG          Set to T if running in debug mode (optional).  Work datasets will not be deleted 
               in debug mode.  This is useful if you are editing the code or want to further 
               manipulate the resulting data sets.  The default value is F.

***********************************************************************************************
For more details, please see the related documentation
**********************************************************************************************/

%MACRO UNI_LOGREG(DATASET=, OUTCOME=, EVENT=, DESC=F, CLIST=, CREFLIST=, NLIST=, MATCHID=,type3=T, 
     FIRTH=F, PO=F, DOC=T, OUTPATH=, FNAME=, ORIENTATION=PORTRAIT, DEBUG=F); 

   %local OUTPATH TYPE3P I CVAR OUTCOME progress DOC OR DATASET __MACRO_ERR WORK_SETS OR_LB
     EVENT REFVAR N NVNAME N_OBS CLIST OUTCOMEVV TYPE3 FNAME DEBUG CLIST_CNT CREFLIST NLIST C
     NVAR OR_UB ORP ORIENTATION out_cat_cnt MODEL MAX MIN RESP out_type;

   /* Prevent case sensitivity */
   %let DEBUG = %UPCASE(&DEBUG);
   %let type3 = %UPCASE(&type3);
   %let desc = %UPCASE(&desc);
   %let firth = %UPCASE(&firth);
   %let po = %UPCASE(&po);
   %let doc = %UPCASE(&doc);
   %let matchid = %UPCASE(&matchid);

   /* Initialize error flag */
   %let __Macro_Err = 0;

   /* Get list of data sets in work library to avoid deletion later */
   ODS OUTPUT Members(nowarn)=_DataSetList;
   PROC DATASETS lib=work memtype=data;
   QUIT;

   /* If there are data sets in the work library */
   %if %sysfunc(exist(_DataSetList)) %then %do;
      PROC SQL noprint;
         select Name
         into :work_sets separated by ' '
         from _DataSetList;
      quit;
   %end;
   %else %do;
      %let work_sets =;
   %end;

   /* Save current options */
   PROC OPTSAVE out=_options;
   RUN;

   /* Count number of variables */
   %if &clist = %STR() %then %do; 
      %let clist_cnt = 0; 
   %end;
   %else %do;
      %let clist_cnt = %sysfunc(countw(&CLIST));
   %end;

   /* Get number of outcome levels */
   /* Have to use proc freq not proc sql because sql does not collapse formatted values */
   PROC FREQ DATA = &dataset noprint;
      tables &outcome/out=_freq;
   RUN;
   PROC SQL noprint;
      select count(*) into :out_cat_cnt
      from _freq
      /* Don't count missing values */
      where MISSING(&outcome) = 0;
   QUIT;

   /* Find out if outcome variable is character or numeric */
   DATA _NULL_;
      set &dataset;
      CALL SYMPUT('out_type',VTYPE(&outcome));
   RUN;

   /* Check for outcome variable names that are too long */
   %DO i = 1 %to &clist_cnt; 
      %IF %LENGTH(%SCAN(&CLIST, &i)) > 30 %then %do;
         %put ERROR: Variable name %SCAN(&CLIST, &i) must be less than 31 characters.; 
           %let __Macro_Err=1;
       %END;
   %END;

   /* Make sure PO was not set to T if inappropriate */
   %if &out_cat_cnt <= 2 and &po = T %then %do;
      %put ERROR: PO=T, but the outcome variable only has 2 levels.  The proportional odds test cannot be conducted for binary logit models; 
      %let __Macro_Err=1;
   %end;

   /* Make sure outcome is numeric for cumulative logit models */
   %if &out_cat_cnt > 2 and &out_type = C %then %do;
      %put ERROR: The outcome variable must be numeric for cumulative logit models.; 
      %let __Macro_Err=1;
   %end;

   %if &__Macro_Err. %then %do;
      data _null_;
         abort 3;
      run;
   %end;

   *CHARACTER Variables;
   %IF &CLIST NE  %THEN %DO; 
      %LET C = 1; 

      %DO %UNTIL (%SCAN(&CLIST, &C) =   ); 
         %LET CVAR = %SCAN(&CLIST, &C); 
         %LET REFVAR = %SCAN(&CREFLIST, &C, *); 

         /* If no reference categories were provided then use the default */
         %if %sysevalf(%superq(REFVAR)=,boolean) %then %do;
            %let refvar = &cvar;
         %end;

         ODS OUTPUT  "Parameter Estimates" = _mlec "Type 3 Tests" = _type3 
          "95% Clodds=Wald" = _or  ResponseProfile=_resp ModelInfo=_info
           %if &po = T %then %do; CumulativeModelTest=_score %end;;
         proc logistic data=&dataset;
            /* If cumulative logit model then remove formats from outcome otherwise problems */
            /* will be caused when determining the response order */
            %if &out_cat_cnt > 2 %then %do; FORMAT &OUTCOME; %end;
            class &REFVAR / param=ref order=internal;
            /* Note that if EVENT and DESC are specified that EVENT will override DESC */
            /* EVENT will be ignored if link=clogit */
            model &OUTCOME (%if %sysevalf(%superq(event)~=,boolean) %then %do; 
               event=&event %end; order=internal %if &desc = T %then %do; desc %end;) = &CVAR/
               clodds=wald %if &firth = T %then %do; firth %end;;
			%if &matchid ~=%str() %then %do; strata &matchid %end;;
         run;
          
         ODS OUTPUT 'One-Way Frequencies' = FREQ; 
         PROC FREQ DATA=&dataset ORDER=data; 
            TABLE &CVAR;
            WHERE &OUTCOME is not missing;
         RUN;  
           
         DATA _FREQC; 
            length Covariate $256. Level $96.;
            SET FREQ ;
            Covariate = left(label(&CVAR)); 
            Level = left(F_&CVAR); 
            Variable = substr(Table,7,96);
            KEEP Covariate Level Frequency Variable; 
         RUN;

         data _type3_; 
            length effect $256;
            set _type3;
            rename ProbChiSq = type3p effect=Variable; 
            format ProbChiSq;
            keep effect ProbChiSq;
         run;

         proc sort data=_type3_; by Variable; run; 
         proc sort data=_FREQC; by Variable; run; 

         data _mg; 
            merge _type3_ _FREQC; 
            by Variable; 
            keep Covariate Variable Level Frequency type3p; 
         run; 

         data _mlec_; 
            set _mlec; 
            format ProbChiSq; 
            where Variable ne 'Intercept'; 
            morder = _n_; 
         run;

         data _comb; 
            merge _mlec_ (rename=(ClassVal0=Level)) _or (rename=(OddsRatioEst=OddsRatio)); 
            keep Variable Level ProbChiSq OddsRatio LowerCL UpperCL morder; 
         run; 
 
         proc sort data=_comb;by Variable Level;run;
         proc sort data=_mg; by Variable Level; run;

         data _mergedata; 
            length Covariate $256. Level $96. ;
            merge _mg _comb; 
            by Variable Level; 
            if morder = . then morder = 999;
            /* Preserve variable order */
            order2 = &c;
            keep order2 Covariate Level Frequency OddsRatio LowerCL UpperCL ProbChiSq morder type3p; 
         run; 
                    
         /* Merge on score test */
         %if &PO = T %then %do;
            data _mergedata; 
               set _mergedata;
               if _n_ = 1 then set _score (KEEP=ProbChiSq RENAME=(ProbChiSq=PO_Pval));
            RUN;
         %end;

         proc sort data=_mergedata out=_charr&C (DROP=morder); 
            by Covariate morder; 
         run;  
                                   
         %LET C = %EVAL(&C+1);
      %END; 
      %LET C = %eval(&C - 1) ;

      data _charr_all; 
         set _charr1 - _charr&C; 
         order = 1;
      run;
   %END;

   *NUMERIC Variables;
   %IF &NLIST NE  %THEN %DO; 
          
      %LET N = 1; 

      %DO %UNTIL (%SCAN(&NLIST, &N) =   ); 
         %LET NVAR = %SCAN(&NLIST, &N); 
         ODS OUTPUT  "Parameter Estimates" = _mlec "Global Tests" = _wald "95% Clodds=Wald" = _or
          "Observations summary" = _nobs  ResponseProfile=_resp ModelInfo=_info
          %if &po = T %then %do; CumulativeModelTest=_score %end;;
         proc logistic data=&dataset;
            /* If cumulative logit model then remove formats from outcome otherwise problems */
            /* will be caused when determining the response order */
            %if &out_cat_cnt > 2 %then %do; FORMAT &OUTCOME; %end;
            /* Note that if EVENT and DESC are specified that EVENT will override DESC */
            /* EVENT will be ignored if link=clogit */
            model &OUTCOME (%if %sysevalf(%superq(event)~=,boolean) %then %do; event=&event %end; 
               %if &desc = T %then %do; desc %end;
               order=internal)= &NVAR/clodds=wald %if &firth = T %then %do; firth %end;;
         run; 

         data _mlec;
            set _mlec; 
            format ProbChiSq; 
            where Variable ne 'Intercept'; 
         run;

         data _wald;
            set _wald; 
            format ProbChiSq; 
         run;

         data _NULL_; 
            set &dataset; 
            call symput('NVNAME', put(label(&NVAR),$256.));
         run;

         proc sql noprint;
            select ProbChiSq into: type3p from _wald where test ="Wald";
            select OddsRatioEst into: or from _or;
            select LowerCL into: or_lb from _or;
            select UpperCL into: or_ub from _or;
            select ProbChiSq into: orp from _mlec;
            select SumFreqsUsed into: N_obs from _nobs;
         quit;

         data _numm&N; 
            length Covariate $256.;
            Covariate = "&NVNAME";
            Level = ' ';
            Frequency = &N_obs;
            type3p = &type3p;
            LowerCL = &or_lb;
            UpperCL = &or_ub;
            OddsRatio = &or;
            ProbChiSq = &orp;
            /* Preserve order */
            order2 = &n;
            keep order2 Covariate Frequency Level type3p OddsRatio LowerCL UpperCL ProbChiSq; 
         run;
                    
         /* Merge on score test */
         %if &PO = T %then %do;
            data _numm&N; 
               set _numm&N;
               if _n_ = 1 then set _score (KEEP=ProbChiSq RENAME=(ProbChiSq=PO_Pval));
            RUN;
         %end;

         %LET N = %EVAL(&N+1);
      %END; 
      %LET N = %EVAL(&N-1);

      data _numm_all; 
         set _numm1 - _numm&N;
         order = 2; 
      run; 
   %END;

   /* Combine categorical and numeric covariate results */
   DATA _report;
      set %IF &CLIST NE %then %do; _charr_all %end; 
         %if &NLIST NE %then %do; _numm_all %end;;

      /* OR and 95% CI */
      if OddsRatio ~= . then 
         OR_CI = TRIM(LEFT(PUT(OddsRatio,8.2))) || " (" || 
         TRIM(LEFT(PUT(lowerCL,8.2))) || "-" || 
         TRIM(LEFT(PUT(upperCL,8.2))) || ")";
      else OR_CI = '-';
   RUN;

   /* Get model type - binary logit or cumulative logit */
   PROC SQL noprint;
      select Value into :model
      from _info
      where Description = 'Model';
   QUIT;

   /* For cumulative logit models find out the direction of the outcome */
   %if &model = %QUOTE(cumulative logit) %then %do;
      PROC SQL noprint;
         select Outcome into :max
         from _resp
         where OrderedValue = 1;
         select Outcome into :min
         from _resp
         having OrderedValue = max(orderedvalue);
      QUIT;

      %if &max > &min %then %let resp = higher;
      %else %if &min > &max %then %let resp = lower;
   %end;

*---- table template -----;  
ODS PATH WORK.TEMPLAT(UPDATE)
SASUSR.TEMPLAT(UPDATE) SASHELP.TMPLMST(READ);

PROC TEMPLATE;
DEFINE STYLE STYLES.TABLES;
NOTES "MY TABLE STYLE"; 
PARENT=STYLES.MINIMAL;

     STYLE SYSTEMTITLE /FONT_SIZE = 12pt     FONT_FACE = "TIMES NEW ROMAN";

     STYLE HEADER /
           FONT_FACE = "TIMES NEW ROMAN"
            CELLPADDING=8
            JUST=C
            VJUST=C
            FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD; 

     STYLE TABLE /
            FRAME=HSIDES            /* outside borders: void, box, above/below, vsides/hsides, lhs/rhs */
           RULES=GROUP              /* internal borders: none, all, cols, rows, groups */
           CELLPADDING=6            /* the space between table cell contents and the cell border */
            CELLSPACING=6           /* the space between table cells, allows background to show */
            JUST=C
            FONT_SIZE = 10pt
           BORDERWIDTH = 0.5pt;  /* the width of the borders and rules */

     STYLE DATAEMPHASIS /
           FONT_FACE = "TIMES NEW ROMAN"
           FONT_SIZE = 10pt
           FONT_WEIGHT = BOLD;

     STYLE DATA /
           FONT_FACE = "TIMES NEW ROMAN" 
           FONT_SIZE = 10pt;

     STYLE SYSTEMFOOTER /FONT_SIZE = 9pt FONT_FACE = "TIMES NEW ROMAN" JUST=C;
END;

RUN; 

   /* Remove outer quotes for report */
   %if %sysevalf(%superq(event)~=,boolean) %then %do;
      %let event = %sysfunc(DEQUOTE(&event));
   %end;

   **** PRINT THE TABLE ************;
   data _NULL_; 
      set &dataset; 
      call symputx('outcomevv', label(&OUTCOME)); 
   run;

   OPTIONS ORIENTATION=&ORIENTATION MISSING = "-" NODATE;

   %if &doc = T %then %do;
      ODS RTF STYLE=TABLES file= "&OUTPATH.&FNAME &SYSDATE..DOC";  
   %end;

   PROC REPORT DATA=_report HEADLINE HEADSKIP CENTER STYLE(REPORT)={JUST=CENTER} SPLIT='~'
     nowd SPANROWS LS=256; 
      COLUMNS order order2 Covariate 
          %if &CLIST ~= %STR() %then %do; Level %end; 
          Frequency (
          %if %sysevalf(%superq(event)~=,boolean) %then %do; "&outcomevv=&event" %end;
          %else %do; "&outcomevv" %end;
               '------------------------------------------'(OR_CI ProbChiSq  
          %if &type3 = T %then %do; type3p %end;
          %if &PO = T %then %do; PO_pval %end;)); 
      DEFINE order/order order=internal noprint;
      DEFINE order2/order order=internal noprint;
      DEFINE Covariate/ order order=Data  "Covariate"  STYLE(COLUMN) = {JUST = L}; 
      %if &CLIST ~= %STR() %then %do; 
         DEFINE Level/ DISPLAY   "Level"     STYLE(COLUMN) = {JUST =L}; 
      %end;
      DEFINE Frequency/DISPLAY "N" STYLE(COLUMN) = {JUST = C};                
      DEFINE OR_CI/DISPLAY "Odds Ratio~(95% CI)" STYLE(COLUMN) = {JUST = C CellWidth=15%}; 
      DEFINE ProbChiSq/DISPLAY  "OR P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%} FORMAT=PVALUE5.3; 

      COMPUTE ProbChiSq; 
         IF  . < ProbChiSq <0.05 THEN CALL DEFINE("ProbChiSq", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
      ENDCOMP; 

      %if &type3 = T %then %do;
         DEFINE type3p/order "Type3 P-value" STYLE(COLUMN) = {JUST = C CellWidth=8%} FORMAT=PVALUE5.3; 
         COMPUTE type3p; 
            IF  . < type3p <0.05 THEN CALL DEFINE("type3p", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
         ENDCOMP; 
      %end;

      %if &PO = T %then %do;
         DEFINE PO_pval/order "Assumption P-value" STYLE(COLUMN) = {JUST = C CellWidth=9%} FORMAT=PVALUE5.3; 
         COMPUTE PO_pval; 
            IF  . < PO_pval <0.05 THEN CALL DEFINE("PO_pval", "STYLE", "STYLE=[FONT_WEIGHT=BOLD]"); 
         ENDCOMP; 
      %end;

      COMPUTE after Covariate; line ''; ENDCOMP; 

      /* For cumulative logit models add a footnote */
      %if &model = %QUOTE(cumulative logit) %then %do;
         compute after _page_;            
           line @0 "The probability of having &resp values of the outcome is being modeled.";
         ENDCOMP;
      %end;
   RUN;

   %if &doc = T %then %do;
      ODS RTF CLOSE;
   %end;

   /* Reload original options that were in use before running the macro */
   /* This is mainly to reset the orientation */
   PROC OPTLOAD data=_options;
   RUN;

   /* Only delete files if not in debug mode */
   %if &DEBUG ~= T %then %do;

      /* If there are work data sets that should not be deleted */
      %if %sysevalf(%superq(work_sets)~=,boolean) %then %do;
         /* DELETE ALL TEMPORARY DATASETS that were created */
         proc datasets lib=work memtype=data;  
            save &work_sets;
          quit;  
       %end;
       %else %do;
         proc datasets lib=work kill memtype=data;  
         quit; 
      %end;
   %end;

%MEND UNI_LOGREG; 
